/*************************************************************************************
*	irrlamb - http://irrlamb.googlecode.com
*	Copyright (C) 2010  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef CAMERA_H
#define CAMERA_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Classes
class CameraClass {

	public:

		CameraClass();
		~CameraClass();

		void Update(const vector3df &Target);
		void RecordReplay();
		void HandleMouseMotion(float UpdateX, float UpdateY);

		void SetRotation(float Yaw, float Pitch) { this->Yaw = Yaw, this->Pitch = Pitch; }
		void SetDistance(float Distance) { MaxDistance = Distance; }

		float GetYaw() const { return Yaw; }
		float GetPitch() const { return Pitch; }

		void SetYaw(float Value) { Yaw = Value; }
		void SetPitch(float Value) { Pitch = Value; }
		const vector3df &GetDirection() const { return Direction; }

		ICameraSceneNode *GetNode() { return Node; }

	private:

		ICameraSceneNode *Node;
		matrix4 Transform;
		vector3df Direction;
		float Yaw, Pitch, MaxDistance, Distance;

		// Replays
		vector3df PreviousPosition, PreviousLookAt;
		bool MovementChanged;

};

#endif
