/*************************************************************************************
*	irrlamb - http://irrlamb.googlecode.com
*	Copyright (C) 2010  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef GRAPHICS_H
#define GRAPHICS_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Structures
struct VideoModeStruct {

	int Width, Height, BPP;
	stringw String;
};

// Classes
class GraphicsClass {

	public:

		int Init(int Width, int Height, bool Fullscreen, E_DRIVER_TYPE DriverType, IEventReceiver *EventReceiver);
		int Close();

		void BeginFrame();
		void EndFrame();

		void SaveScreenshot();

		void SetClearColor(const SColor &Color) { ClearColor = Color; }
		void SetDrawScene(bool Value) { DrawScene = Value; }
		void ToggleCursor(bool Value) { ShowCursor = Value; }

		const array<VideoModeStruct> &GetVideoModes() { return VideoModes; }
		u32 GetCurrentVideoModeIndex();

	private:

		void CreateScreenshot();

		// Graphics state
		SColor ClearColor;
		bool DrawScene, ShowCursor;

		// Screenshots
		bool ScreenshotRequested;

		// Modes
		array<VideoModeStruct> VideoModes;
};

// Singletons
typedef SingletonClass<GraphicsClass> Graphics;

#endif
