/*************************************************************************************
*	irrlamb - http://irrlamb.googlecode.com
*	Copyright (C) 2010  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef PHYSICS_H
#define PHYSICS_H

// Libraries
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include "singleton.h"

// Classes
class PhysicsClass {

	public:

		enum FilterTypes {
			FILTER_NONE			= 0x0,
			FILTER_RIGIDBODY	= 0x1,
	        FILTER_STATIC		= 0x2,
	        FILTER_KINEMATIC	= 0x4,
			FILTER_GRAPPLE		= 0x8,
	        FILTER_CAMERA		= 0x10,
			FILTER_ZONE			= 0x20,
			FILTER_BASICBODIES	= FILTER_RIGIDBODY | FILTER_STATIC | FILTER_KINEMATIC,
		};

		int Init();
		int Close();

		void Update(float FrameTime);
		void Reset();

		bool RaycastWorld(const btVector3 &Start, btVector3 &End, btVector3 &ormal);
		void QuaternionToEuler(const btQuaternion &Quat, btVector3 &Euler);

		btDiscreteDynamicsWorld *GetWorld() { return World; }

		void SetEnabled(bool Value) { Enabled = Value; }
		bool IsEnabled() const { return Enabled; }
		void RemoveFilter(int &Value, int Filter);
		void SetBodyType(int &Value, int Filter);

	private:

		btDefaultCollisionConfiguration *CollisionConfiguration;
		btCollisionDispatcher *Dispatcher;
		btBroadphaseInterface *BroadPhase;
		btSequentialImpulseConstraintSolver *Solver;
		btDiscreteDynamicsWorld *World;

		bool Enabled;
};

// Singletons
typedef SingletonClass<PhysicsClass> Physics;

#endif
