/*************************************************************************************
*	irrlamb - http://irrlamb.googlecode.com
*	Copyright (C) 2010  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef SAVE_H
#define SAVE_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>
#include <map>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations
class DatabaseClass;

// Struct for one highscore
struct HighScoreStruct {
	HighScoreStruct() : Time(0), DateStamp(0) { }
	HighScoreStruct(float Time, int DateStamp) : Time(Time), DateStamp(DateStamp) { }

	bool operator<(const HighScoreStruct &Value) {
		return Time < Value.Time;
	}

	float Time;
	time_t DateStamp;
};

// Struct for one level stat
struct SaveLevelStruct {
	SaveLevelStruct() : ID(0), Unlocked(0), LoadCount(0), LoseCount(0), WinCount(0), PlayTime(0) { }

	int ID;
	int Unlocked;
	int LoadCount, LoseCount, WinCount;
	float PlayTime;
	array<HighScoreStruct> HighScores;
};

// Classes
class SaveClass {

	public:

		int Init();
		int Close();

		int InitStatsDatabase();

		void GetSavePath(const stringc &File, stringc &Path) { Path = SavePath + File; }
		void GetReplayPath(const stringc &File, stringc &Path) { Path = ReplayPath + File; }
		void GetScreenshotsPath(const stringc &File, stringc &Path) { Path = ScreenshotsPath + File; }
		void GetConfigFile(const stringc &File, stringc &Path) { Path = ConfigFile + File; }
		const stringc &GetReplayPath() { return ReplayPath; }
		const stringc &GetConfigFile() { return ConfigFile; }
		const stringc &GetScreenshotsPath() { return ScreenshotsPath; }

		int LoadLevelStats();
		void SaveLevelStats(const stringc &Level);
		void UpdateLevelStats(const stringc &Level, const SaveLevelStruct &Stats);
		bool GetLevelStats(const stringc &Level, SaveLevelStruct &Stats);
		const SaveLevelStruct *GetLevelStats(const stringc &Level);

		void AddScore(const stringc &Level, float Time);
		void IncrementLevelLoadCount(const stringc &Level);
		void IncrementLevelLoseCount(const stringc &Level);
		void IncrementLevelWinCount(const stringc &Level);
		void IncrementLevelPlayTime(const stringc &Level, float Time);
		void UnlockLevel(const stringc &Level);

	private:

		// Paths
		stringc SavePath, ReplayPath, ScreenshotsPath, ConfigFile, StatsFile;
		
		// Stats
		std::map<stringc, SaveLevelStruct> LevelStats;
		std::map<stringc, SaveLevelStruct>::iterator LevelStatsIterator;

		// Database
		DatabaseClass *Database;
};

// Singletons
typedef SingletonClass<SaveClass> Save;

#endif
