/*************************************************************************************
*	irrlamb - http://irrlamb.googlecode.com
*	Copyright (C) 2010  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef SCRIPTING_H
#define SCRIPTING_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include <lua.hpp>
#include <map>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Structures
struct TimedCallbackStruct {
	float TimeStamp;
	stringc Function;
};

// Forward Declarations
class ObjectClass;

// Classes
class ScriptingClass {

	public:

		ScriptingClass();
		~ScriptingClass() { }

		int Init();
		int Close();

		void Reset();
		int LoadFile(const stringc &File);

		void DefineLuaVariable(const char *VariableName, const char *Value);

		void CallFunction(const stringc &FunctionName);
		void CallCollisionHandler(const stringc &FunctionName, ObjectClass *BaseObject, ObjectClass *OtherObject);
		void CallZoneHandler(const stringc &FunctionName, int Type, ObjectClass *Zone, ObjectClass *Object);

		bool HandleKeyPress(EKEY_CODE Key);
		void HandleMousePress(int Button, int MouseX, int MouseY);
		void UpdateTimedCallbacks();

	private:

		static luaL_reg CameraFunctions[], ObjectFunctions[], OrbFunctions[], TimerFunctions[], LevelFunctions[],
						GUIFunctions[], RandomFunctions[], ZoneFunctions[];

		static bool CheckArguments(lua_State *LuaObject, int Required);

		static int CameraSetYaw(lua_State *LuaObject);
		static int CameraSetPitch(lua_State *LuaObject);

		static int ObjectGetPointer(lua_State *LuaObject);
		static int ObjectGetName(lua_State *LuaObject);
		static int ObjectSetPosition(lua_State *LuaObject);
		static int ObjectGetPosition(lua_State *LuaObject);
		static int ObjectStop(lua_State *LuaObject);
		static int ObjectSetAngularVelocity(lua_State *LuaObject);
		static int ObjectSetLifetime(lua_State *LuaObject);
		static int ObjectDelete(lua_State *LuaObject);

		static int OrbDeactivate(lua_State *LuaObject);

		static int TimerDelayedFunction(lua_State *LuaObject);
		static int TimerStamp(lua_State *LuaObject);

		static int LevelLose(lua_State *LuaObject);
		static int LevelWin(lua_State *LuaObject);
		static int LevelGetTemplate(lua_State *LuaObject);
		static int LevelCreateObject(lua_State *LuaObject);
		static int LevelCreateConstraint(lua_State *LuaObject);
		static int LevelCreateSpring(lua_State *LuaObject);

		static int GUITutorialText(lua_State *LuaObject);

		static int RandomSeed(lua_State *LuaObject);
		static int RandomGetFloat(lua_State *LuaObject);
		static int RandomGetInt(lua_State *LuaObject);
		
		void AddTimedCallback(const stringc &FunctionName, float Time);
		void AttachKeyToFunction(EKEY_CODE Key, const stringc &FunctionName);
		
		list<TimedCallbackStruct> TimedCallbacks;

		std::map<EKEY_CODE, stringc> KeyCallbacks;
		std::map<EKEY_CODE, stringc>::iterator KeyCallbacksIterator;

		lua_State *LuaObject;

};

// Singletons
typedef SingletonClass<ScriptingClass> Scripting;

#endif
